"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DerivativesOrdersAccount = void 0;
const types_1 = require("../types");
const mango_client_1 = require("@blockworks-foundation/mango-client");
const anchor_1 = require("@project-serum/anchor");
const utils_1 = require("../utils");
const aaob_1 = require("@chugach-foundation/aaob");
const shared_1 = require("../constants/shared");
class DerivativesOrdersAccount {
    constructor(client, address, state, _onStateUpdate) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        _onStateUpdate && this.subscribe();
    }
    static async load(client, address, onStateUpdateHandler) {
        const state = (await client.accounts.ordersAccount.fetchNullable(address));
        return new DerivativesOrdersAccount(client, address, state, onStateUpdateHandler);
    }
    getOrderIds() {
        const orderIds = [];
        for (const order of this.state.openOrders) {
            if (order.orderId != mango_client_1.ZERO_BN) {
                orderIds.push(order.orderId);
            }
        }
        return orderIds;
    }
    getOrdersMap() {
        const orders = new Map();
        for (const order of this.state.openOrders) {
            if (order.orderId != mango_client_1.ZERO_BN) {
                orders.set(order.orderId.toString(), order);
            }
        }
        return orders;
    }
    async getOrders(viewer) {
        const orders = [];
        const ordersMap = this.getOrdersMap();
        const openOrdersAddress = this.address;
        const { bids, asks } = await viewer.getOrderBookFull();
        for (const bid of bids.items(false)) {
            const order = ordersMap.get(bid.leafNode.key.toString());
            if (order) {
                const { price, quantity } = this.getPriceQuantity(viewer, bid.leafNode);
                orders.push({
                    side: types_1.Side.Bid,
                    price,
                    quantity,
                    subAccountIdx: order.subAccountIdx,
                    orderType: order.orderType,
                    timestamp: order.timestamp,
                    clientOrderId: order.clientOrderId,
                    orderId: order.orderId,
                    openOrdersAddress
                });
            }
        }
        for (const ask of asks.items(false)) {
            const order = ordersMap.get(ask.leafNode.key.toString());
            if (order) {
                const { price, quantity } = this.getPriceQuantity(viewer, ask.leafNode);
                orders.push({
                    side: types_1.Side.Ask,
                    price,
                    quantity,
                    subAccountIdx: order.subAccountIdx,
                    orderType: order.orderType,
                    timestamp: order.timestamp,
                    clientOrderId: order.clientOrderId,
                    orderId: order.orderId,
                    openOrdersAddress
                });
            }
        }
        return orders;
    }
    getPriceQuantity(viewer, leafNode) {
        const baseM = viewer.market.state.inner.baseMultiplier;
        const baseQty = new anchor_1.BN(leafNode.baseQuantity.toNumber()).mul(baseM);
        const price = (0, utils_1.splToUiAmount)((0, utils_1.priceLotsToNative)((0, aaob_1.getPriceFromKey)(leafNode.key), viewer.market.state.inner.baseMultiplier, viewer.market.state.inner.quoteMultiplier, viewer.market.state.inner.config.decimals), shared_1.QUOTE_TOKEN_DECIMALS);
        const quantity = (0, utils_1.splToUiAmount)(baseQty, viewer.market.state.inner.config.decimals);
        return { price, quantity };
    }
    getBaseTokenFree(subAcctIdx, baseDecimals) {
        return (0, utils_1.splToUiAmount)(this.state.baseTokenFree[subAcctIdx], baseDecimals);
    }
    getQuoteTokenFree(subAcctIdx, quoteDecimals) {
        return (0, utils_1.splToUiAmount)(this.state.quoteTokenFree[subAcctIdx], quoteDecimals);
    }
    subscribe() {
        this.client.accounts.ordersAccount
            .subscribe(this.address)
            .on('change', (state) => {
            this.state = state;
            // todo: check if dexMarkets need to be reloaded.(market listing/delisting)
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        });
    }
    async unsubscribe() {
        await this.client.accounts.ordersAccount.unsubscribe(this.address);
    }
}
exports.DerivativesOrdersAccount = DerivativesOrdersAccount;
//# sourceMappingURL=ordersAccount.js.map