"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pool = void 0;
const web3_js_1 = require("@solana/web3.js");
const serum_1 = require("@project-serum/serum");
const spl_token_1 = require("@solana/spl-token");
const utils_1 = require("../utils");
const mango_client_1 = require("@blockworks-foundation/mango-client");
class Pool {
    constructor(client, address, state, market, _onStateUpdate) {
        this.client = client;
        this.address = address;
        this.state = state;
        this.market = market;
        this._onStateUpdate = _onStateUpdate;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, clearing, cacheAccount, tokenMint, authority, dexMarket, oracleProducts, args) {
        const [pool, poolBump] = (0, utils_1.derivePoolAddress)(args.poolName, client.cypherPID);
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const [poolNode, poolNodeBump] = (0, utils_1.derivePoolNodeAddress)(pool, 0, client.cypherPID);
        const [tokenVault] = (0, utils_1.derivePoolNodeVaultAddress)(poolNode, client.cypherPID);
        args.poolBump = poolBump;
        await client.methods
            .createPool(args)
            .accounts({
            clearing,
            cacheAccount,
            pool,
            tokenMint,
            tokenVault,
            oracleProducts,
            dexMarket,
            authority,
            payer: client.walletPubkey,
            systemProgram: web3_js_1.SystemProgram.programId,
            tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
            rent: web3_js_1.SYSVAR_RENT_PUBKEY
        })
            .rpc();
        return pool;
    }
    static async load(client, address, onStateUpdateHandler) {
        const state = (await client.accounts.pool.fetchNullable(address));
        if (!state.tokenMint.equals(client.quoteMint)) {
            const market = await serum_1.Market.load(client.connection, state.dexMarket, {}, client.dexPID);
            return new Pool(client, address, state, market, onStateUpdateHandler);
        }
        return new Pool(client, address, state, null, onStateUpdateHandler);
    }
    static async loadAll(client) {
        const pools = [];
        const queryResult = await client.accounts.pool.all();
        for (const result of queryResult) {
            if (!result.account.tokenMint.equals(client.quoteMint)) {
                const market = await serum_1.Market.load(client.connection, result.account.dexMarket, {}, client.dexPID);
                pools.push(new Pool(client, result.publicKey, result.account, market));
            }
            else {
                pools.push(new Pool(client, result.publicKey, result.account));
            }
        }
        return pools;
    }
    get dexMarket() {
        return this.state.dexMarket;
    }
    get maxApr() {
        return mango_client_1.I80F48.fromNumber(this.state.config.maxApr);
    }
    get optimalApr() {
        return mango_client_1.I80F48.fromNumber(this.state.config.optimalApr);
    }
    get optimalUtil() {
        return mango_client_1.I80F48.fromNumber(this.state.config.optimalUtil);
    }
    get deposits() {
        return new mango_client_1.I80F48(this.state.deposits);
    }
    get borrows() {
        return new mango_client_1.I80F48(this.state.borrows);
    }
    get totalDeposits() {
        return this.deposits.mul(this.depositIndex);
    }
    get totalBorrows() {
        return this.borrows.mul(this.borrowIndex);
    }
    get depositIndex() {
        return new mango_client_1.I80F48(this.state.depositIndex);
    }
    get borrowIndex() {
        return new mango_client_1.I80F48(this.state.borrowIndex);
    }
    get depositRate() {
        const deposits = this.totalDeposits;
        const borrows = this.totalBorrows;
        const borrowRate = this.borrowRate;
        if (deposits.isZero() && borrows.isZero()) {
            return mango_client_1.ZERO_I80F48;
        }
        else if (deposits.isZero()) {
            return this.maxApr;
        }
        const utilization = borrows.mul(mango_client_1.I80F48.fromNumber(100)).div(deposits);
        return borrowRate.mul(utilization).div(mango_client_1.I80F48.fromNumber(100));
    }
    get borrowRate() {
        const deposits = this.totalDeposits;
        const borrows = this.totalBorrows;
        const { optimalUtil, optimalApr, maxApr } = this;
        if (deposits.isZero() && borrows.isZero()) {
            return mango_client_1.ZERO_I80F48;
        }
        if (borrows.gte(deposits)) {
            return this.maxApr;
        }
        const utilization = borrows.mul(mango_client_1.I80F48.fromNumber(100)).div(deposits);
        if (utilization.gte(optimalUtil)) {
            const extraUtil = utilization.sub(optimalUtil);
            const slope = maxApr
                .sub(optimalApr)
                .div(mango_client_1.I80F48.fromNumber(100).sub(optimalUtil));
            return optimalApr.add(slope.mul(extraUtil));
        }
        const slope = optimalApr.div(optimalUtil);
        return slope.mul(utilization);
    }
    subscribe() {
        this.client.accounts.pool
            .subscribe(this.address)
            .on('change', (state) => {
            this.state = state;
            // todo: check if dexMarkets need to be reloaded.(market listing/delisting)
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        });
    }
    async unsubscribe() {
        await this.client.accounts.pool.unsubscribe(this.address);
    }
}
exports.Pool = Pool;
//# sourceMappingURL=pool.js.map